package de.hft.stuttgart.citydoctor2.webservice.database;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;

import org.hibernate.Session;
import org.hibernate.query.NativeQuery;
import org.hibernate.query.Query;
import org.hibernate.search.jpa.FullTextEntityManager;
import org.hibernate.search.jpa.FullTextQuery;
import org.hibernate.search.jpa.Search;
import org.hibernate.search.query.dsl.QueryBuilder;

import de.hft.stuttgart.citydoctor2.webservice.model.BuildingReference;
import de.hft.stuttgart.citydoctor2.webservice.model.BuildingReferenceWithErrorIndicator;
import de.hft.stuttgart.citydoctor2.webservice.model.entities.WebBuilding;

public class BuildingQueries {

	private BuildingQueries() {
		// only static use
	}

	public static void persistBuilding(WebBuilding buildingRef, EntityManager manager) {
		manager.getTransaction().begin();
		try {
			manager.persist(buildingRef);
		} finally {
			manager.getTransaction().commit();
		}
	}

	public static List<BuildingReference> getBuildingsForModel(int from, int modelId, EntityManager manager) {
		Session session = manager.unwrap(Session.class);
		try {
			session.getTransaction().begin();
			Query<BuildingReference> query = session.createQuery("SELECT new de.hft.stuttgart.citydoctor2."
					+ "webservice.model.BuildingReference(b.id, b.gmlId, b.modelWidth) FROM WebBuilding b WHERE model.id = ?1",
					BuildingReference.class);
			query.setParameter(1, modelId);
			query.setMaxResults(20);
			query.setFirstResult(from);
			return query.getResultList();
		} finally {
			session.getTransaction().commit();
		}
	}
	
	public static List<String> getBuildingsForSearchString(String s, int modelId, EntityManager manager) {
		FullTextEntityManager textEntityManager = Search.getFullTextEntityManager(manager);
		textEntityManager.getTransaction().begin();
		try {
			QueryBuilder queryBuilder = textEntityManager.getSearchFactory().buildQueryBuilder().forEntity(WebBuilding.class).get();
			org.apache.lucene.search.Query luceneQ = queryBuilder.keyword().onField("gmlId").matching(s).createQuery();
			FullTextQuery query = textEntityManager.createFullTextQuery(luceneQ, WebBuilding.class);
			query.setMaxResults(5);
			@SuppressWarnings("unchecked")
			List<WebBuilding> resultList = query.getResultList();
			for (WebBuilding b : resultList) {
				System.out.println(b.getGmlId());
			}
			return null;
		} finally {
			textEntityManager.getTransaction().commit();
		}
	}

	public static WebBuilding getBuildingForGmlId(String gmlId, int modelId, EntityManager manager) {
		manager.getTransaction().begin();
		try {
			TypedQuery<WebBuilding> query = manager.createQuery("FROM WebBuilding WHERE model.id = ?1 AND gmlId = ?2",
					WebBuilding.class);
			query.setParameter(1, modelId);
			query.setParameter(2, gmlId);
			return query.getSingleResult();
		} finally {
			manager.getTransaction().commit();
		}
	}

	public static long getBuildingCountForModel(int modelId, EntityManager manager) {
		manager.getTransaction().begin();
		try {
			javax.persistence.Query query = manager
					.createQuery("SELECT COUNT (b.id) FROM WebBuilding b WHERE b.model.id = ?1");
			query.setParameter(1, modelId);
			return (Long) query.getSingleResult();
		} finally {
			manager.getTransaction().commit();
		}
	}

	public static List<BuildingReference> getBuildingsWithErrorIndicator(int from, int modelId,
			Integer validationId, EntityManager manager) {
		Session session = manager.unwrap(Session.class);
		try {
			session.getTransaction().begin();
			@SuppressWarnings("unchecked")
			NativeQuery<Object[]> nQuery = session.createSQLQuery(
					"SELECT b.id, b.gml_id, b.model_width, CASE WHEN e.id IS NULL THEN FALSE ELSE TRUE END "
					+ "FROM building_reference b "
					+ "LEFT JOIN (SELECT br.id FROM building_reference br WHERE EXISTS "
					+ "(SELECT 1 FROM weberror ee WHERE ee.building_id = br.id AND ee.validation_id = ?2)) AS e ON (e.id = b.id) "
					+ "WHERE b.model_id = ?1");
			nQuery.setParameter(1, modelId);
			nQuery.setParameter(2, validationId);
			nQuery.setFirstResult(from);
			nQuery.setMaxResults(20);
			List<Object[]> list = nQuery.list();
			List<BuildingReference> result = new ArrayList<>();
			for (Object[] objects : list) {
				result.add(new BuildingReferenceWithErrorIndicator((Integer) objects[0], (String) objects[1], (Double) objects[2], (Boolean) objects[3]));
			}
			return result;
		} finally {
			session.getTransaction().commit();
		}
	}
	
	public static long getErrorBuildingCountForModel(int modelId, Integer validationId, EntityManager manager) {
		Session session = manager.unwrap(Session.class);
		session.getTransaction().begin();
		try {
			@SuppressWarnings("unchecked")
			NativeQuery<Object> nQuery = session.createSQLQuery(
					"SELECT COUNT(b.id) "
					+ "FROM building_reference b "
					+ "JOIN (SELECT br.id FROM building_reference br WHERE EXISTS "
					+ "(SELECT 1 FROM weberror ee WHERE ee.building_id = br.id AND ee.validation_id = ?2)) AS e ON (e.id = b.id) "
					+ "WHERE b.model_id = ?1");
			nQuery.setParameter(1, modelId);
			nQuery.setParameter(2, validationId);
			
			return ((BigInteger) nQuery.getSingleResult()).longValueExact();
		} finally {
			session.getTransaction().commit();
		}
	}
	
	public static List<BuildingReference> getErrorBuildingsWithErrorIndicator(int from, int modelId,
			Integer validationId, EntityManager manager) {
		Session session = manager.unwrap(Session.class);
		try {
			session.getTransaction().begin();
			@SuppressWarnings("unchecked")
			NativeQuery<Object[]> nQuery = session.createSQLQuery(
					"SELECT b.id, b.gml_id, b.model_width, TRUE "
					+ "FROM building_reference b "
					+ "JOIN (SELECT br.id FROM building_reference br WHERE EXISTS "
					+ "(SELECT 1 FROM weberror ee WHERE ee.building_id = br.id AND ee.validation_id = ?2)) AS e ON (e.id = b.id) "
					+ "WHERE b.model_id = ?1");
			nQuery.setParameter(1, modelId);
			nQuery.setParameter(2, validationId);
			nQuery.setFirstResult(from);
			nQuery.setMaxResults(20);
			List<Object[]> list = nQuery.list();
			List<BuildingReference> result = new ArrayList<>();
			for (Object[] objects : list) {
				result.add(new BuildingReferenceWithErrorIndicator((Integer) objects[0], (String) objects[1], (Double) objects[2], (Boolean) objects[3]));
			}
			return result;
		} finally {
			session.getTransaction().commit();
		}
	}

}
